@@ -33,6 +33,7 @@
 #include "terminal-widget.h"
 #include "terminal-profile.h"
 #include "terminal.h"
+#include "skey-popup.h"
 #include <libgnome/gnome-util.h> /* gnome_util_user_shell */
 #include <libgnome/gnome-url.h> /* gnome_url_show */
 #include <string.h>
@@ -56,6 +57,7 @@ struct _TerminalScreenPrivate
   GtkWidget *title_entry;
   char *working_dir;
   int child_pid;
+  double font_scale;
   guint recheck_working_dir_idle;
 };
 
@@ -164,6 +166,8 @@ terminal_screen_init (TerminalScreen *screen)
   screen->priv->recheck_working_dir_idle = 0;
   
   screen->priv->term = terminal_widget_new ();
+
+  screen->priv->font_scale = PANGO_SCALE_MEDIUM;
   
   g_object_ref (G_OBJECT (screen->priv->term));
   gtk_object_sink (GTK_OBJECT (screen->priv->term));
@@ -386,6 +390,15 @@ reread_profile (TerminalScreen *screen)
   terminal_widget_set_scrollback_lines (term,
                                         terminal_profile_get_scrollback_lines (profile));
 
+  if (terminal_profile_get_use_skey (screen->priv->profile))
+    {
+      terminal_widget_skey_match_add (screen->priv->term,
+				      "s/key ([0-9]*) ([-A-Za-z0-9]*)");
+    }
+  else
+    {
+      terminal_widget_skey_match_remove (screen->priv->term);
+    }
   bg_type = terminal_profile_get_background_type (profile);
   
   if (bg_type == TERMINAL_BACKGROUND_IMAGE)
@@ -502,6 +515,9 @@ update_color_scheme (TerminalScreen *screen)
                               &fg, &bg, palette);
 }
 
+/* Note this can be called on style_set, on prefs changing,
+ * and on setting the font scale factor
+ */
 static void
 terminal_screen_update_on_realize (GtkWidget      *term,
                                    TerminalScreen *screen)
@@ -539,20 +555,33 @@ terminal_screen_update_on_realize (GtkWidget      *term,
                                                    pango_font_description_get_size (term->style->font_desc));
                 }
 
-              terminal_widget_set_pango_font (term,
-                                              terminal_profile_get_font (profile));
+              pango_font_description_set_size (desc,
+                                               screen->priv->font_scale *
+                                               pango_font_description_get_size (desc));
+              
+              terminal_widget_set_pango_font (term, desc);
 
               pango_font_description_free (desc);
             }
         }
       else
         {
-          terminal_widget_set_pango_font (term,
-                                          terminal_profile_get_font (profile));
+          PangoFontDescription *desc;
+
+          desc = pango_font_description_copy (terminal_profile_get_font (profile));
+          pango_font_description_set_size (desc,
+                                           screen->priv->font_scale *
+                                           pango_font_description_get_size (desc));          
+          
+          terminal_widget_set_pango_font (term, desc);
+
+          pango_font_description_free (desc);
         }
     }
   else
     {
+      /* font_scale is not supported in X fonts mode */
+      
       GdkFont *font;
 
       font = NULL;
@@ -975,7 +1004,7 @@ new_window_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              NULL,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0);
 }
 
 static void
@@ -985,7 +1014,7 @@ new_tab_callback (GtkWidget      *menu_item,
   terminal_app_new_terminal (terminal_app_get (),
                              terminal_profile_get_for_new_term (screen->priv->profile),
                              screen->priv->window,
-                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL);
+                             FALSE, FALSE, NULL, NULL, NULL, NULL, NULL, 1.0);
 }
 
 static void
@@ -1187,6 +1216,7 @@ terminal_screen_do_popup (TerminalScreen *screen,
                           GdkEventButton *event)
 {
   GtkWidget *profile_menu;
+  GtkWidget *im_menu;
   GtkWidget *menu_item;
   GList *profiles;
   GList *tmp;
@@ -1292,6 +1322,21 @@ terminal_screen_do_popup (TerminalScreen *screen,
 		   screen);
 
   
+  im_menu = gtk_menu_new ();
+  menu_item = gtk_menu_item_new_with_mnemonic (_("_Input Methods"));
+
+  gtk_menu_item_set_submenu (GTK_MENU_ITEM (menu_item),
+                             im_menu);
+
+  terminal_widget_im_append_menuitems (screen->priv->term,
+		  		       GTK_MENU_SHELL (im_menu));
+
+  gtk_widget_show (im_menu);
+  gtk_widget_show (menu_item);
+
+  gtk_menu_shell_append (GTK_MENU_SHELL (screen->priv->popup_menu),
+                         menu_item);
+
   if (terminal_window_get_menubar_visible (screen->priv->window))
     menu_item = append_menuitem (screen->priv->popup_menu,
 				 _("Hide _Menubar"),
@@ -1358,7 +1403,25 @@ terminal_screen_button_press_event (GtkWidget      *widget,
     terminal_widget_check_match (term,
                                  event->x / char_width,
                                  event->y / char_height);
-  
+
+  if (terminal_profile_get_use_skey (screen->priv->profile))
+    {
+      gchar *skey_match;
+
+      skey_match = terminal_widget_skey_check_match (term,
+						     event->x / char_width,
+						     event->y / char_height);
+      if (skey_match != NULL &&
+	  event->button == 1 &&
+	  event->state & GDK_CONTROL_MASK)
+	{
+	  terminal_skey_do_popup (screen, GTK_WINDOW (terminal_screen_get_window (screen)), skey_match);
+	  g_free (skey_match);
+
+	  return TRUE;
+	}
+    }
+
   if (event->button == 1 || event->button == 2)
     {
       gtk_widget_grab_focus (widget);
@@ -1505,6 +1568,37 @@ queue_recheck_working_dir (TerminalScreen *screen)
     }
 }
 
+
+void
+terminal_screen_set_font_scale (TerminalScreen *screen,
+                                double          factor)
+{
+  g_return_if_fail (TERMINAL_IS_SCREEN (screen));
+
+  if (factor < TERMINAL_SCALE_MINIMUM)
+    factor = TERMINAL_SCALE_MINIMUM;
+  if (factor > TERMINAL_SCALE_MAXIMUM)
+    factor = TERMINAL_SCALE_MAXIMUM;
+  
+  screen->priv->font_scale = factor;
+  
+  if (screen->priv->term &&
+      GTK_WIDGET_REALIZED (screen->priv->term))
+    {
+      /* Update the font */
+      terminal_screen_update_on_realize (screen->priv->term,
+                                         screen);
+    }
+}
+
+double
+terminal_screen_get_font_scale (TerminalScreen *screen)
+{
+  g_return_val_if_fail (TERMINAL_IS_SCREEN (screen), 1.0);
+  
+  return screen->priv->font_scale;
+}
+
 static void
 terminal_screen_widget_title_changed (GtkWidget      *widget,
                                       TerminalScreen *screen)
@@ -1702,15 +1796,14 @@ drag_data_received  (GtkWidget *widget, GdkDragContext *context,
         
         str = gtk_selection_data_get_text (selection_data);
 
-        /* FIXME should this be converted back to locale encoding
-         * before sending to the terminal? Most likely yes.
+        /*
+	 * pass UTF-8 to the terminal widget. The terminal widget
+	 * should know which encoding mode it's in and be able
+	 * to perform the correct conversion.
          */
         if (str && *str)
-          {
-            terminal_widget_write_data_to_child (screen->priv->term,
-                                                 str, strlen (str));
-          }
-
+	  terminal_widget_write_data_to_child (screen->priv->term,
+					       str, strlen (str));
         g_free (str);
       }
       break;
@@ -2144,8 +2237,6 @@ make_font_monospace (const PangoFontDescription *font)
 {
   PangoFontDescription *ret;
 
-  return NULL;
-  
   ret = pango_font_description_copy (font);
 
   /* FIXME */
