@@ -29,6 +29,7 @@
 typedef struct
 {
   int foo;
+  int skey_tag;
 } VteData;
 
 static void
@@ -111,6 +112,29 @@ terminal_widget_match_add                  (GtkWidget            *widget,
   vte_terminal_match_add(VTE_TERMINAL(widget), regexp);
 }
 
+void
+terminal_widget_skey_match_add             (GtkWidget            *widget,
+					    const char           *regexp)
+{
+  VteData *data;
+  
+  data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
+
+  data->skey_tag = vte_terminal_match_add(VTE_TERMINAL(widget), regexp);
+}
+
+void
+terminal_widget_skey_match_remove          (GtkWidget            *widget)
+{
+  VteData *data;
+  
+  data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
+
+  vte_terminal_match_remove(VTE_TERMINAL(widget), data->skey_tag);
+
+  data->skey_tag = -1;
+}
+
 char*
 terminal_widget_check_match (GtkWidget *widget,
 			     int        column,
@@ -119,6 +143,26 @@ terminal_widget_check_match (GtkWidget *widget,
   return vte_terminal_match_check(VTE_TERMINAL(widget), column, row, NULL);
 }
 
+char*
+terminal_widget_skey_check_match (GtkWidget *widget,
+				  int        column,
+				  int        row)
+{
+  VteData *data;
+  gint tag;
+  char *match;
+   
+  data = g_object_get_data (G_OBJECT (widget), "terminal-widget-data");
+
+  match = vte_terminal_match_check(VTE_TERMINAL(widget), column, row, &tag);
+  if (data->skey_tag == tag)
+    return match;
+
+  g_free (match);
+  return NULL;
+      
+}
+
 void
 terminal_widget_set_word_characters (GtkWidget  *widget,
                                      const char *str)
@@ -275,7 +319,8 @@ void
 terminal_widget_set_allow_bold (GtkWidget *widget,
 				gboolean   setting)
 {
-  UNIMPLEMENTED;
+  g_return_if_fail(VTE_IS_TERMINAL(widget));
+  vte_terminal_set_allow_bold(VTE_TERMINAL(widget), setting);
 }
 
 void
@@ -431,7 +476,8 @@ terminal_widget_fork_command (GtkWidget   *widget,
 			      GError     **err)
 {
   *child_pid = vte_terminal_fork_command (VTE_TERMINAL (widget),
-		 			  path, argv, envp);
+		 			  path, argv, envp, working_dir,
+					  update_records, TRUE, TRUE);
   return (*child_pid != -1);
 }
 
@@ -495,3 +541,9 @@ terminal_widget_supports_dynamic_encoding (void)
 {
   return TRUE;
 }
+
+void
+terminal_widget_im_append_menuitems(GtkWidget *widget, GtkMenuShell *menushell)
+{
+  vte_terminal_im_append_menuitems(VTE_TERMINAL(widget), menushell);
+}
