@@ -358,6 +358,12 @@ terminal_widget_match_add (GtkWidget            *widget,
                       VTATTR_UNDERLINE, NULL);  
 }
 
+void
+terminal_widget_skey_match_add (GtkWidget            *widget,
+				const char           *regexp)
+{
+}
+
 char*
 terminal_widget_check_match (GtkWidget            *widget,
                              int                   column,
@@ -367,6 +373,19 @@ terminal_widget_check_match (GtkWidget            *widget,
                                          column, row, 0));
 }
 
+char*
+terminal_widget_skey_check_match (GtkWidget            *widget,
+				  int                   column,
+				  int                   row)
+{
+	return NULL;
+}
+
+void
+terminal_widget_skey_match_remove (GtkWidget            *widget)
+{
+}
+
 void
 terminal_widget_set_word_characters (GtkWidget  *widget,
                                      const char *str)
@@ -1105,3 +1124,30 @@ terminal_widget_supports_pango_fonts (void)
 {
   return FALSE;
 }
+
+const char*
+terminal_widget_get_encoding (GtkWidget *widget)
+{
+  const char *charset = NULL;
+  g_get_charset (&charset);
+  return charset;
+}
+
+void
+terminal_widget_set_encoding (GtkWidget  *widget,
+                              const char *encoding)
+{
+  ; /* does nothing */
+}
+
+gboolean
+terminal_widget_supports_dynamic_encoding (void)
+{
+  return FALSE;
+}
+
+void
+terminal_widget_im_append_menuitems(GtkWidget *widget, GtkMenuShell *menushell)
+{
+  ; /* does nothing */
+}
